import argparse
import asyncio
import discord
import discord.ext.commands as commands
import discord.app_commands as app_commands

from astra.connections import AstraDBConnection
from astra.groups import *

class Astra(commands.Bot):
    def __init__(self, prefix, **kwargs):
        super().__init__(prefix, **kwargs)
        
    async def setup_hook(self) -> None:
        await super().setup_hook()
        AstraDBConnection.initialize()
        await self.tree.sync()
        
    async def on_ready(self):
        print('Ready!')
        
    @app_commands.command(description='Version, changelog, and other info')
    async def info(self, interaction: discord.Interaction):
        await interaction.response.send_message(
            """
            **ASTRA: SE**
            A Simple Text Recording Assistant: Slash Edition v1.2

            **New features:**
            - QUOTE GENERATION IS **BACK**. Run `/quote gen` to EXPERIENCE THE MAGIC.""", ephemeral=True)
        
if __name__ == '__main__':
    def main():
        parser = argparse.ArgumentParser()
        parser.add_argument('token')
        args = parser.parse_args()
    
        intents = discord.Intents.default()
        intents.message_content = True
        astra = Astra('!', intents=intents)
        asyncio.run(astra.add_cog(DebugGroup(astra)))
        asyncio.run(astra.add_cog(QuoteGroup(astra)))
        astra.run(args.token)
        
    main()


import discord

from astra.connections import AstraDBConnection
from astra.generation import AstraMarkovModel

class QuoteView(discord.ui.View):
    
    def __init__(self, interaction: discord.Interaction, target: discord.Member | discord.User, raw: list[tuple[str, int]], perPage: int = 5):
        self.interaction = interaction
        self._raw = raw
        self.total_pages = (len(raw) // perPage) + 1 if (len(raw) % perPage != 0) else (len(raw) // perPage)
        self.page = 1
        self.per_page = perPage
        self.target = target
        super().__init__(timeout=60)
        
    async def show(self):
        embed = await self.build_view()
        await self.update_buttons()
        await self.interaction.response.send_message(embed=embed, view=self)
        
    async def build_view(self):
        start, end = (self.page - 1) * self.per_page, self.page * self.per_page
        embed = discord.Embed(title=self.target, description='')
        for (msg, ind) in self._raw[start:end]:
            embed.description += f'#{ind}: "{msg}"\n\n'
        embed.set_author(name=f'Requested by {self.interaction.user}')
        embed.set_footer(text=f'Page {self.page} of {self.total_pages}')
        embed.set_thumbnail(url=self.target.display_avatar.url)
        return embed
    
    async def update_view(self, interaction: discord.Interaction):
        embed = await self.build_view()
        await self.update_buttons()
        await interaction.response.edit_message(embed=embed, view=self)
        
    async def update_buttons(self):
        self.children[0].disabled = (self.total_pages == 1 or self.page <= 2)
        self.children[1].disabled = (self.total_pages == 1 or self.page == 1)
        self.children[2].disabled = (self.total_pages == 1 or self.page == self.total_pages)
        self.children[3].disabled = (self.total_pages == 1 or self.page >= self.total_pages - 1)
        
    async def interaction_check(self, interaction: discord.Interaction) -> bool:
        return interaction.user.id == self.interaction.user.id
       
    @discord.ui.button(style=discord.ButtonStyle.secondary, emoji='⏮️')
    async def first(self, interaction: discord.Interaction, button: discord.Button):
        self.page = 1
        await self.update_view(interaction)
    
    @discord.ui.button(style=discord.ButtonStyle.primary, emoji='⏪')
    async def prev(self, interaction: discord.Interaction, button: discord.Button):
        self.page -= 1
        await self.update_view(interaction)
        
    @discord.ui.button(style=discord.ButtonStyle.primary, emoji='⏩')
    async def succ(self, interaction: discord.Interaction, button: discord.Button):
        self.page += 1
        await self.update_view(interaction)
        
    @discord.ui.button(style=discord.ButtonStyle.secondary, emoji='⏭️')
    async def last(self, interaction: discord.Interaction, button: discord.Button):
        self.page = self.total_pages
        await self.update_view(interaction)
        
    async def on_timeout(self):
        message = await self.interaction.original_response()
        await message.edit(view=None)
    
    

class AstraHandler:
    
    @staticmethod
    async def add_quote(interaction: discord.Interaction, user: discord.Member | discord.User, msg: str):
        if isinstance(msg, discord.Message):
            msg = msg.clean_content
        if await AstraHandler.does_quote_exist(user, msg):
            await interaction.response.send_message('Quote already present.', ephemeral=True)
        else:
            AstraDBConnection().add_quote(user.id, msg)
            await interaction.response.send_message(f'Quoted {user.mention} saying "{msg}"')
            AstraMarkovModel().initialize_model()
        
    @staticmethod
    async def read_quotes(interaction: discord.Interaction, fromUser: discord.Member | discord.User):
        raw = AstraDBConnection.read_quotes(fromUser.id)
        if len(raw) == 0:
            await interaction.response.send_message('No quotes found.', ephemeral=True)
        else:
            view = QuoteView(interaction, fromUser, raw)
            await view.show()
    
    @staticmethod
    async def does_quote_exist(fromUser: discord.Member | discord.User, withMsg: str | discord.Message, /):
        if isinstance(withMsg, discord.Message):
            withMsg = withMsg.clean_content
        result = AstraDBConnection.search_quote(fromUser.id, withMsg)
        print(result)
        return len(result) > 0
    
    @staticmethod
    async def debug_remove_quote(withIdent: int):
        AstraDBConnection.delete_quote(withIdent)


import numpy as np
import markovify

from astra.connections import AstraDBConnection

class AstraMarkovModel:
    _instance = None
    
    def __new__(cls):
        if cls._instance is None:
            AstraDBConnection()
            cls._instance = super().__new__(cls)
            cls._instance.initialize_model()
        return cls._instance
        
    def initialize_model(self):
        all_quotes = np.array(AstraDBConnection.query_all())
        quotes = '\n'.join(all_quotes)
        self._model = markovify.NewlineText(quotes).compile()
        
    def make_sentence(self):
        return self._model.make_sentence()


import sqlite3

def _connect():
    con = sqlite3.connect('data/astra.db')
    cur = con.cursor()
    return con, cur

class AstraDBConnection:

    @staticmethod
    def initialize():
        con, cur = _connect()
        cur.execute(
            """CREATE TABLE IF NOT EXISTS quotes(
                id INTEGER PRIMARY KEY,
                user INTEGER NOT NULL,
                msg TEXT NOT NULL
            )"""
        )
        con.commit()
        con.close()
    
    @staticmethod
    def add_quote(user: int, msg: str):
        con, cur = _connect()
        cur.execute('insert into quotes values(NULL, ?, ?)', (user, msg))
        con.commit()
        con.close()
        
    @staticmethod
    def search_quote(fromUser: int, withMsg: str):
        con, cur = _connect()
        res = cur.execute('select id from quotes where user = ? and msg = ?', (fromUser, withMsg)).fetchall()
        con.close()
        return res
    
    @staticmethod
    def read_quotes(fromUser: int):
        con, cur = _connect()
        res = cur.execute('select msg, id from quotes where user=? order by id desc', (fromUser,)).fetchall()
        con.close()
        return res
    
    @staticmethod
    def delete_quote(withId: int):
        con, cur = _connect()
        cur.execute('delete from quotes where id = ?', (withId,))
        con.commit()
        con.close()
        
    @staticmethod
    def query_all():
        con, cur = _connect()
        res = cur.execute('select msg from quotes').fetchall()
        return [v[0] for v in res]